@@ -26,14 +26,25 @@ rtrunc <- function(distr=runif, n, linf=-Inf, lsup=Inf,...)
 #For discrete dictribution, rtrunc sample within \samp{(linf, lsup]}. See example.
 #NOTE
 #The inversion of the quantile function leads to time consuming functions for some distributions.
+#WARNING: The method is flexible, but can lead to problems linked to rounding errors in some extreme situations.
+#The function checks that the values are in the expected range and returns an error if not.
+#It also warns some extreme situation that could lead to unexpected results.
+#See Examples.
 #EXAMPLE
 #rtrunc("rnorm", n=10, linf=0)
 #range(rtrunc(rnorm, n=1000, linf=3, lsup=5, sd=10))
 ### Discrete distributions
 #range(rtrunc(rpois,1000,linf=2,lsup=4,lambda=1))
+###Examples of rounding problems. 
+###The first one will provide a warning while the results are unexpected, 
+###The second will provide an error.
+#\dontrun{
+#table(rtrunc(rbinom, n=1000, size=10, prob=1-1E-20, lsup=9))
+#table(rtrunc(rbinom, n=1000, size=10, prob=1E-14, linf=0))
+#}
 
 #CREATED 08-02-20
-#REVISED 10-02-10
+#REVISED 13-10-01
 #--------------------------------------------
 {
     linf <- as.vector(linf)
@@ -48,6 +59,7 @@ rtrunc <- function(distr=runif, n, linf=-Inf, lsup=Inf,...)
     pinf <- as.vector(pfun(q=linf,...))
     psup <- as.vector(pfun(q=lsup,...))
 
+    
     p <- runif(n,min=pinf,max=psup)
 
     qfun <- get(paste("q",distr,sep=""),mode="function")
@@ -58,6 +70,11 @@ rtrunc <- function(distr=runif, n, linf=-Inf, lsup=Inf,...)
     res[pinf <= 0 & res > lsup] <- NaN          #ex: rtrunc("lnorm",10,linf=-2,lsup=-1)
     res[psup>=1 & res < linf] <- NaN          #ex: rtrunc("unif",10,linf=2,lsup=4,max=1)
     res[is.na(linf) | is.na(lsup)] <- NaN   #ex: rtrunc("norm",10,sd=-2)
-
+    
+    
+    #Two tests for extreme situations. None Catch all possibilities. THe error is first to avoid the warning
+    if(any(res <= linf | res > lsup, na.rm=TRUE)) stop("Error in rtrunc: some values are not in the expected range (maybe due to rounding errors)")
+    if(isTRUE(all.equal(pinf,1)) | isTRUE(all.equal(psup,0)) ) warning("Warning: check the results from rtrunc. It may have reached rounding errors")
+    
     return(res)
 }
